#include "ndematerialstyle.h"
#include <QDebug>
#include <QColor>
#include <QMetaEnum>
#include <QPalette>
#include <ndesettings.h>
#include "ndematerialstyle_p.h"
namespace Nde
{

static const QRgb colorScheme[] = {0xff2c9dfc, 0xff011222, 0xff222222, 0xffffffff};
static const QRgb whiteMaskColor = 0xffffffff;
static const QRgb blackMaskColor = 0xff000000;

QColor overlayColors(const QColor &base, const QColor &paint, QPainter::CompositionMode comp)
{
    // This isn't the fastest way, but should be "fast enough".
    // It's also the only safe way to use QPainter::CompositionMode
    QImage img(1, 1, QImage::Format_ARGB32_Premultiplied);
    QPainter p(&img);
    QColor start = base;
    start.setAlpha(255); // opaque
    p.fillRect(0, 0, 1, 1, start);
    p.setCompositionMode(comp);
    p.fillRect(0, 0, 1, 1, paint);
    p.end();
    return img.pixel(0, 0);
}
QColor alphaColor(QColor &base, qreal alpha)
{
    if (alpha >= 0 && alpha < 1.0) {
        base.setAlphaF(alpha);
    }
    return base;
}
template <typename Enum>
static Enum toEnumValue(const QByteArray &value, bool *ok)
{
    QMetaEnum enumeration = QMetaEnum::fromType<Enum>();
    return static_cast<Enum>(enumeration.keyToValue(value.constData(), ok));
}
MaterialStylePrivate::MaterialStylePrivate(MaterialStyle *parent):mTheme(MaterialStyle::Light)
  ,mPrimary(QColor::fromRgba(colorScheme[MaterialStyle::Primary]))
  ,mSecondary(QColor::fromRgba(colorScheme[MaterialStyle::Secondary]))
  ,mWindowText(QColor::fromRgba(colorScheme[MaterialStyle::WindowText]))
  ,mWindow(QColor::fromRgba(colorScheme[MaterialStyle::Window]))
  ,settings(new Settings(QStringLiteral("nde")))
  ,q_ptr(parent)
{
    updateSettings();
    connect(settings.data(), &Settings::settingsChanged, [=]()
    {
        updateSettings();
    });
}
void MaterialStylePrivate::updateSettings()
{
    if(settings->childGroups().contains(QStringLiteral("Style")))
    {
        settings->beginGroup(QStringLiteral("Style"));
        QByteArray themeValue = settings->value(QStringLiteral("Theme")).toByteArray();
        bool ok = false;
        MaterialStyle::Theme themeEnum = toEnumValue<MaterialStyle::Theme>(themeValue, &ok);
        if(ok)
            mTheme = themeEnum;
        QString primaryValue = settings->value(QStringLiteral("Primary")).toString();
        mPrimary = QColor(primaryValue).isValid() ? QColor(primaryValue) :
                                                    QColor::fromRgba(colorScheme[MaterialStyle::Primary]);

        QString secondaryValue = settings->value(QStringLiteral("Secondary")).toString();
        mSecondary = QColor(secondaryValue).isValid() ? QColor(secondaryValue) :
                                                        QColor::fromRgba(colorScheme[MaterialStyle::Secondary]);

        QString windowTextValue = settings->value(QStringLiteral("WindowText")).toString();
        mWindowText = QColor(windowTextValue).isValid() ? QColor(windowTextValue) : QColor::fromRgba(colorScheme[MaterialStyle::WindowText]);

        QString windowValue = settings->value(QStringLiteral("Window")).toString();
        mWindow = QColor(windowValue).isValid() ? QColor(windowValue) : QColor::fromRgba(colorScheme[MaterialStyle::Window]);
        settings->endGroup();
    }
}
QColor MaterialStylePrivate::brush(MaterialStyle::ColorSet name, qreal opacity, MaterialStyle::MaskType type)
{
    QColor maskColor;
    switch(type){
    case MaterialStyle::WhiteMask:
        maskColor = QColor::fromRgba(whiteMaskColor);
        break;
    case MaterialStyle::BlackMask:
        maskColor = QColor::fromRgba(blackMaskColor);
        break;
    case MaterialStyle::PrimaryMask:
        maskColor = mPrimary;
        break;
    }
    maskColor.setAlphaF(opacity);
    return overlayColors(QColor::fromRgba(colorScheme[name]), maskColor);
}

QColor MaterialStylePrivate::brush(MaterialStyle::ColorSet name, QColor color)
{
    color.setAlpha(255);
    return overlayColors(QColor::fromRgba(colorScheme[name]), color);
}
QColor MaterialStylePrivate::brush(MaterialStyle::ColorSet name, qreal opacity)
{
    QColor color = QColor::fromRgba(colorScheme[name]);
    return alphaColor(color, opacity);
}
MaterialStylePrivate::~MaterialStylePrivate()
{

}
void MaterialStylePrivate::createPalette(QPalette &palette)
{
    QColor base = brush(Nde::MaterialStyle::Secondary, 0.06);
    QColor alternateBase = brush(Nde::MaterialStyle::Secondary, 0.1);
    QColor shadow = brush(Nde::MaterialStyle::Secondary, 0.02);

    palette.setColor(QPalette::All,      QPalette::Window,          mWindow);
    palette.setColor(QPalette::All,      QPalette::WindowText,      mWindowText);
    palette.setColor(QPalette::All,      QPalette::Base,            base);
    palette.setColor(QPalette::All,      QPalette::AlternateBase,   alternateBase);
    palette.setColor(QPalette::All,      QPalette::ToolTipBase,     mSecondary);
    palette.setColor(QPalette::All,      QPalette::ToolTipText,     mWindowText);
    palette.setColor(QPalette::All,      QPalette::Text,            mWindowText);
    palette.setColor(QPalette::All,      QPalette::Button,          base);
    palette.setColor(QPalette::All,      QPalette::ButtonText,      mWindowText);
    palette.setColor(QPalette::All,      QPalette::BrightText,      mWindowText);

    palette.setColor(QPalette::All,      QPalette::Light,           QColor("white"));
    palette.setColor(QPalette::All,      QPalette::Midlight,        QColor("#d7d7d7"));
    palette.setColor(QPalette::All,      QPalette::Mid,             QColor("#b4b4b4"));
    palette.setColor(QPalette::All,      QPalette::Dark,            QColor("#1a1a1a"));
    palette.setColor(QPalette::All,      QPalette::Shadow,          shadow);

    palette.setColor(QPalette::All,      QPalette::Highlight,       mPrimary);
    palette.setColor(QPalette::All,      QPalette::HighlightedText, mWindowText);

    palette.setColor(QPalette::All,      QPalette::Link,            QColor("#5151ff"));
    palette.setColor(QPalette::All,      QPalette::LinkVisited,     QColor("#5151ff"));

    QColor insensitiveFgColor = QColor("#bababa");
    palette.setColor(QPalette::Disabled, QPalette::Window,          mWindow);
    palette.setColor(QPalette::Disabled, QPalette::WindowText,      insensitiveFgColor);
    palette.setColor(QPalette::Disabled, QPalette::Base,            base);
    palette.setColor(QPalette::Disabled, QPalette::AlternateBase,   alternateBase);
    palette.setColor(QPalette::Disabled, QPalette::Text,            insensitiveFgColor);
    palette.setColor(QPalette::Disabled, QPalette::Button,          base);
    palette.setColor(QPalette::Disabled, QPalette::ButtonText,      insensitiveFgColor);
    palette.setColor(QPalette::Disabled, QPalette::BrightText,      insensitiveFgColor);

    palette.setColor(QPalette::Disabled, QPalette::Light,           QColor("#f4f4f4"));
    palette.setColor(QPalette::Disabled, QPalette::Midlight,        QColor("#f4f4f4"));
    palette.setColor(QPalette::Disabled, QPalette::Dark,            QColor("#f4f4f4"));
    palette.setColor(QPalette::Disabled, QPalette::Mid,             QColor("#c3c3c3"));
    palette.setColor(QPalette::Disabled, QPalette::Shadow,          shadow);

    palette.setColor(QPalette::Disabled, QPalette::Highlight,       mPrimary);
    palette.setColor(QPalette::Disabled, QPalette::HighlightedText, insensitiveFgColor);

    palette.setColor(QPalette::Disabled, QPalette::Link,            QColor("#5151ff"));
    palette.setColor(QPalette::Disabled, QPalette::LinkVisited,     QColor("#5151ff"));


    palette.setColor(QPalette::Inactive, QPalette::Window,          mWindow);
    palette.setColor(QPalette::Inactive, QPalette::WindowText,      insensitiveFgColor);
    palette.setColor(QPalette::Inactive, QPalette::Base,            base);
    palette.setColor(QPalette::Inactive, QPalette::AlternateBase,   alternateBase);
    palette.setColor(QPalette::Inactive, QPalette::Text,            insensitiveFgColor);
    palette.setColor(QPalette::Inactive, QPalette::Button,          base);
    palette.setColor(QPalette::Inactive, QPalette::ButtonText,      insensitiveFgColor);
    palette.setColor(QPalette::Inactive, QPalette::BrightText,      mWindowText);

    palette.setColor(QPalette::Inactive, QPalette::Light,           QColor("white"));
    palette.setColor(QPalette::Inactive, QPalette::Midlight,        QColor("#d7d7d7"));
    palette.setColor(QPalette::Inactive, QPalette::Mid,             QColor("#b4b4b4"));
    palette.setColor(QPalette::Inactive, QPalette::Dark,            QColor("#33393b"));
    palette.setColor(QPalette::Inactive, QPalette::Shadow,          shadow);

    palette.setColor(QPalette::Inactive, QPalette::Highlight,       mPrimary);
    palette.setColor(QPalette::Inactive, QPalette::HighlightedText, insensitiveFgColor);

    palette.setColor(QPalette::Inactive, QPalette::Link,            QColor("#5151ff"));
    palette.setColor(QPalette::Inactive, QPalette::LinkVisited,     QColor("#5151ff"));
}
MaterialStyle::MaterialStyle(QObject *parent) : QObject(parent)
  ,d_ptr(new MaterialStylePrivate(this))
{
}

void MaterialStyle::setTheme(Theme theme)
{
    Q_D(MaterialStyle);
    d->mTheme = theme;
}
void MaterialStyle::createPalette(QPalette &palette)
{
    Q_D(MaterialStyle);
    d->createPalette(palette);
}
QColor MaterialStyle::brush(ColorSet name, qreal opacity, MaskType type)
{
    Q_D(MaterialStyle);
    return d->brush(name, opacity, type);
}
QColor MaterialStyle::brush(MaterialStyle::ColorSet name, QColor color)
{
    Q_D(MaterialStyle);
    return d->brush(name, color);
}
QColor MaterialStyle::brush(MaterialStyle::ColorSet name, qreal opacity)
{
    Q_D(MaterialStyle);
    return d->brush(name, opacity);
}
}
